// ignore_for_file: avoid_print

import 'dart:io';

import 'package:flutter/services.dart';
import 'package:logging/logging.dart';
import 'package:path_provider/path_provider.dart';
import 'package:test/test.dart';

Future<bool> _pathProviderFunctionTest(
    String fName, Future Function() func, Logger _logger) async {
  bool ret = true;
  String res = '$fName:';
  try {
    var d = await func();
    res = '$res $d';
  } on UnimplementedError {
    res = '$res <Unimplemented>';
    ret = false;
  } on UnsupportedError {
    res = '$res <Unsupported>';
    ret = false;
  } on MissingPluginException {
    res = '$res <missingPlugin>';
    ret = false;
  } on Exception catch (e) {
    res = '$res <Exception ($e)>';
    ret = false;
  } on Error catch (e) {
    res = '$res <Error ($e)>';
    ret = false;
  }
  _logger.info('path_provider support $res');
  return ret;
}

Future<void> main() async {
  final Logger _logger = Logger('diagnostics');
  Logger.root.level = Level.INFO;
  Logger.root.onRecord.listen((record) {
    print('${record.level.name}: ${record.time}: ${record.message}');
  });

  group('diagnostics', () {
    test('platform support test', () async {
      _logger.info(
          '*************************************************************');
      _logger.info('MacOS:   ${Platform.isMacOS}');
      _logger.info('Linux:   ${Platform.isLinux}');
      _logger.info('Windows: ${Platform.isWindows}');
      _logger.info('Fuchsia: ${Platform.isFuchsia}');
      _logger.info('Android: ${Platform.isAndroid}');
      _logger.info('IOS:     ${Platform.isIOS}');
      _logger.info(
          '*************************************************************');
      _logger.info('Hostname:${Platform.localHostname}');
      _logger.info('Version: ${Platform.version}');
      expect(true, isTrue);
    });

    test('path_provider support test', () async {
      // this is a dummy test providing platform specific diagnostics
      await _pathProviderFunctionTest(
          'getDownloadsDirectory', getDownloadsDirectory, _logger);
      await _pathProviderFunctionTest(
          'getLibraryDirectory', getLibraryDirectory, _logger);
      await _pathProviderFunctionTest(
          'getTemporaryDirectory', getTemporaryDirectory, _logger);
      await _pathProviderFunctionTest('getApplicationSupportDirectory',
          getApplicationSupportDirectory, _logger);
      await _pathProviderFunctionTest('getApplicationDocumentsDirectory',
          getApplicationDocumentsDirectory, _logger);
      await _pathProviderFunctionTest(
          'getExternalStorageDirectory', getExternalStorageDirectory, _logger);
      expect(true, isTrue);
    });
  });
}
