// ignore_for_file: avoid_print

import 'package:dependency_visitor/dependency_visitor.dart';
import 'package:flutter/widgets.dart' as widgets;
import 'package:geiger_localstorage/geiger_localstorage.dart';
import 'package:test/test.dart';

Future<String?> getFile() async {
  String? ret;
  await DependencyVisitor(filePaths: ['lib/assets/store/store.data'])
      .run()
      .forEach((dependencyFile) {
    ret = dependencyFile.content;
  });
  return ret;
}

void main() async {
  group('node_parser tests', () {
    test('availability of asset (1)', () async {
      widgets.WidgetsFlutterBinding.ensureInitialized();
      String? initialNodes = await GenericController.getResourceOrFile(null);
      expect(initialNodes, isNotEmpty,
          reason: 'initial store asset not loaded');
    });
    test('availability of asset (2)', () async {
      String initialNodes =
          await GenericController.getResourceOrFile('store.data');
      expect(initialNodes, isNotEmpty,
          reason: 'initial store asset not loaded');
    });
    test('parse default asset', () async {
      String initialNodes = await GenericController.getResourceOrFile(
          'lib/assets/store/store.data');
      List<Node> ln = await NodeParser(initialNodes).getNodes();
      expect(ln.length > 45, isTrue,
          reason: 'Number of nodes is too low (got only ${ln.length} nodes)');
    });
    test('parse dump database', () async {
      String initialNodes =
          await GenericController('test', DummyMapper('test')).dump();
      List<Node> ln = await NodeParser(initialNodes).getNodes();
      for (Node n in ln) {
        print('## $n');
      }
      expect(ln.length > 50, isTrue,
          reason: 'Number of nodes is too low (got only ${ln.length} nodes)');
    });
    test('parse dumped node with special chars', () async {
      List<Node> initialNodes = [
        await NodeImpl.fromPath(':strangeNode', 'testOwner',
            nodeValues: <NodeValue>[
              NodeValueImpl('test1', '"'),
              NodeValueImpl('test2', '\\'),
              NodeValueImpl('test3', '"\\"'),
              NodeValueImpl('test4', '\\\\')
            ]),
        await NodeImpl.fromPath(':foreignNode', 'testOwner',
            nodeValues: <NodeValue>[
              NodeValueImpl('testJp', 'こんばんわ'),
              NodeValueImpl('testCn', '你好'),
              NodeValueImpl('testFr',
                  'Bonjour! Ca va? Ça y est on e finalement sans COVID!')
            ])
      ];
      String initialNodesText = "";
      for (Node n in initialNodes) {
        initialNodesText += n.toString();
      }
      print('## original $initialNodesText');
      List<Node> ln = await NodeParser(initialNodesText).getNodes();
      String newNodesText = "";
      for (Node n in ln) {
        print('## $n');
        newNodesText += n.toString();
      }
      expect(ln.length, initialNodes.length,
          reason:
              'Number of nodes is not correct (got only ${ln.length} nodes)');
      expect(newNodesText, initialNodesText, reason: 'The nodes do not match');
    });
  });
}
